@@ -38,9 +38,10 @@ import (
 	eventingduckv1alpha1 "knative.dev/eventing/pkg/apis/duck/v1alpha1"
 	eventingduckv1beta1 "knative.dev/eventing/pkg/apis/duck/v1beta1"
 	"knative.dev/eventing/pkg/apis/messaging"
-	"knative.dev/eventing/pkg/apis/messaging/v1alpha1"
-	subscriptionreconciler "knative.dev/eventing/pkg/client/injection/reconciler/messaging/v1alpha1/subscription"
-	listers "knative.dev/eventing/pkg/client/listers/messaging/v1alpha1"
+	v1 "knative.dev/eventing/pkg/apis/messaging/v1"
+	"knative.dev/eventing/pkg/apis/messaging/v1beta1"
+	subscriptionreconciler "knative.dev/eventing/pkg/client/injection/reconciler/messaging/v1beta1/subscription"
+	listers "knative.dev/eventing/pkg/client/listers/messaging/v1beta1"
 	eventingduck "knative.dev/eventing/pkg/duck"
 	"knative.dev/eventing/pkg/logging"
 )
@@ -57,7 +58,8 @@ const (
 )
 
 var (
-	channelGVK = v1alpha1.SchemeGroupVersion.WithKind("Channel")
+	v1beta1ChannelGVK = v1beta1.SchemeGroupVersion.WithKind("Channel")
+	v1ChannelGVK      = v1.SchemeGroupVersion.WithKind("Channel")
 )
 
 func newReconciledNormal(namespace, name string) pkgreconciler.Event {
@@ -87,18 +89,15 @@ var _ subscriptionreconciler.Interface = (*Reconciler)(nil)
 var _ subscriptionreconciler.Finalizer = (*Reconciler)(nil)
 
 // ReconcileKind implements Interface.ReconcileKind.
-func (r *Reconciler) ReconcileKind(ctx context.Context, subscription *v1alpha1.Subscription) pkgreconciler.Event {
-	subscription.Status.InitializeConditions()
-	subscription.Status.ObservedGeneration = subscription.Generation
-
+func (r *Reconciler) ReconcileKind(ctx context.Context, subscription *v1beta1.Subscription) pkgreconciler.Event {
 	// Find the channel for this subscription.
 	channel, err := r.getChannel(ctx, subscription)
 	if err != nil {
-		logging.FromContext(ctx).Warn("Failed to get Spec.Channel as Channelable duck type",
+		logging.FromContext(ctx).Warn("Failed to get Spec.Channel or backing channel as Channelable duck type",
 			zap.Error(err),
 			zap.Any("channel", subscription.Spec.Channel))
-		subscription.Status.MarkReferencesResolvedUnknown(channelReferenceFailed, "Failed to get Spec.Channel as Channelable duck type. %s", err)
-		return newChannelWarnEvent("Failed to get Spec.Channel as Channelable duck type. %s", err)
+		subscription.Status.MarkReferencesResolvedUnknown(channelReferenceFailed, "Failed to get Spec.Channel or backing channel: %s", err)
+		return newChannelWarnEvent("Failed to get Spec.Channel or backing channel: %s", err)
 	}
 
 	// Make sure all the URI's that are suppose to be in status are up to date.
@@ -121,7 +120,7 @@ func (r *Reconciler) ReconcileKind(ctx context.Context, subscription *v1alpha1.S
 	return newReconciledNormal(subscription.Namespace, subscription.Name)
 }
 
-func (r *Reconciler) FinalizeKind(ctx context.Context, subscription *v1alpha1.Subscription) pkgreconciler.Event {
+func (r *Reconciler) FinalizeKind(ctx context.Context, subscription *v1beta1.Subscription) pkgreconciler.Event {
 	channel, err := r.getChannel(ctx, subscription)
 	if err != nil {
 		// If the channel was deleted (i.e., error == notFound), just return nil so that
@@ -138,7 +137,7 @@ func (r *Reconciler) FinalizeKind(ctx context.Context, subscription *v1alpha1.Su
 	return nil
 }
 
-func (r Reconciler) checkChannelStatusForSubscription(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) pkgreconciler.Event {
+func (r Reconciler) checkChannelStatusForSubscription(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) pkgreconciler.Event {
 	ss, err := r.getSubStatus(sub, channel)
 	if err != nil {
 		logging.FromContext(ctx).Warn("Failed to get subscription status.", zap.Error(err))
@@ -158,7 +157,7 @@ func (r Reconciler) checkChannelStatusForSubscription(ctx context.Context, chann
 	return nil
 }
 
-func (r Reconciler) syncChannel(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) pkgreconciler.Event {
+func (r Reconciler) syncChannel(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) pkgreconciler.Event {
 	// Ok, now that we have the Channel and at least one of the Call/Result, let's reconcile
 	// the Channel with this information.
 	if patched, err := r.syncPhysicalChannel(ctx, sub, channel, false); err != nil {
@@ -179,7 +178,7 @@ func (r Reconciler) syncChannel(ctx context.Context, channel *eventingduckv1alph
 	return nil
 }
 
-func (r *Reconciler) resolveSubscriptionURIs(ctx context.Context, subscription *v1alpha1.Subscription) pkgreconciler.Event {
+func (r *Reconciler) resolveSubscriptionURIs(ctx context.Context, subscription *v1beta1.Subscription) pkgreconciler.Event {
 	// Everything that was supposed to be resolved was, so flip the status bit on that.
 	subscription.Status.MarkReferencesResolvedUnknown("Resolving", "Subscription resolution interrupted.")
 
@@ -200,7 +199,7 @@ func (r *Reconciler) resolveSubscriptionURIs(ctx context.Context, subscription *
 	return nil
 }
 
-func (r *Reconciler) resolveSubscriber(ctx context.Context, subscription *v1alpha1.Subscription) pkgreconciler.Event {
+func (r *Reconciler) resolveSubscriber(ctx context.Context, subscription *v1beta1.Subscription) pkgreconciler.Event {
 	// Resolve Subscriber.
 	subscriber := subscription.Spec.Subscriber.DeepCopy()
 	if !isNilOrEmptyDestination(subscriber) {
@@ -227,7 +226,7 @@ func (r *Reconciler) resolveSubscriber(ctx context.Context, subscription *v1alph
 	return nil
 }
 
-func (r *Reconciler) resolveReply(ctx context.Context, subscription *v1alpha1.Subscription) pkgreconciler.Event {
+func (r *Reconciler) resolveReply(ctx context.Context, subscription *v1beta1.Subscription) pkgreconciler.Event {
 	// Resolve Reply.
 	reply := subscription.Spec.Reply.DeepCopy()
 	if !isNilOrEmptyDestination(reply) {
@@ -254,7 +253,7 @@ func (r *Reconciler) resolveReply(ctx context.Context, subscription *v1alpha1.Su
 	return nil
 }
 
-func (r *Reconciler) resolveDeadLetterSink(ctx context.Context, subscription *v1alpha1.Subscription) pkgreconciler.Event {
+func (r *Reconciler) resolveDeadLetterSink(ctx context.Context, subscription *v1beta1.Subscription) pkgreconciler.Event {
 	// Resolve DeadLetterSink.
 	delivery := subscription.Spec.Delivery.DeepCopy()
 	if !isNilOrEmptyDeliveryDeadLetterSink(delivery) {
@@ -282,20 +281,21 @@ func (r *Reconciler) resolveDeadLetterSink(ctx context.Context, subscription *v1
 	return nil
 }
 
-func (r *Reconciler) getSubStatus(subscription *v1alpha1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined) (eventingduckv1alpha1.SubscriberStatus, error) {
+func (r *Reconciler) getSubStatus(subscription *v1beta1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined) (eventingduckv1beta1.SubscriberStatus, error) {
 	if channel.Annotations != nil {
-		if channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1beta1" {
+		if channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1beta1" ||
+			channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1" {
 			return r.getSubStatusV1Beta1(subscription, channel)
 		}
 	}
 	return r.getSubStatusV1Alpha1(subscription, channel)
 }
 
-func (r *Reconciler) getSubStatusV1Alpha1(subscription *v1alpha1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined) (eventingduckv1alpha1.SubscriberStatus, error) {
+func (r *Reconciler) getSubStatusV1Alpha1(subscription *v1beta1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined) (eventingduckv1beta1.SubscriberStatus, error) {
 	subscribableStatus := channel.Status.GetSubscribableTypeStatus()
 
 	if subscribableStatus == nil {
-		return eventingduckv1alpha1.SubscriberStatus{}, fmt.Errorf("channel.Status.SubscribableStatus is nil")
+		return eventingduckv1beta1.SubscriberStatus{}, fmt.Errorf("channel.Status.SubscribableStatus is nil")
 	}
 	for _, sub := range subscribableStatus.Subscribers {
 		if sub.UID == subscription.GetUID() &&
@@ -303,14 +303,14 @@ func (r *Reconciler) getSubStatusV1Alpha1(subscription *v1alpha1.Subscription, c
 			return sub, nil
 		}
 	}
-	return eventingduckv1alpha1.SubscriberStatus{}, fmt.Errorf("subscription %q not present in channel %q subscriber's list", subscription.Name, channel.Name)
+	return eventingduckv1beta1.SubscriberStatus{}, fmt.Errorf("subscription %q not present in channel %q subscriber's list", subscription.Name, channel.Name)
 }
 
-func (r *Reconciler) getSubStatusV1Beta1(subscription *v1alpha1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined) (eventingduckv1alpha1.SubscriberStatus, error) {
+func (r *Reconciler) getSubStatusV1Beta1(subscription *v1beta1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined) (eventingduckv1beta1.SubscriberStatus, error) {
 	for _, sub := range channel.Status.Subscribers {
 		if sub.UID == subscription.GetUID() &&
 			sub.ObservedGeneration == subscription.GetGeneration() {
-			return eventingduckv1alpha1.SubscriberStatus{
+			return eventingduckv1beta1.SubscriberStatus{
 				UID:                sub.UID,
 				ObservedGeneration: sub.ObservedGeneration,
 				Ready:              sub.Ready,
@@ -318,10 +318,10 @@ func (r *Reconciler) getSubStatusV1Beta1(subscription *v1alpha1.Subscription, ch
 			}, nil
 		}
 	}
-	return eventingduckv1alpha1.SubscriberStatus{}, fmt.Errorf("subscription %q not present in channel %q subscriber's list", subscription.Name, channel.Name)
+	return eventingduckv1beta1.SubscriberStatus{}, fmt.Errorf("subscription %q not present in channel %q subscriber's list", subscription.Name, channel.Name)
 }
 
-func (r *Reconciler) trackAndFetchChannel(ctx context.Context, sub *v1alpha1.Subscription, ref corev1.ObjectReference) (runtime.Object, pkgreconciler.Event) {
+func (r *Reconciler) trackAndFetchChannel(ctx context.Context, sub *v1beta1.Subscription, ref corev1.ObjectReference) (runtime.Object, pkgreconciler.Event) {
 	// Track the channel using the channelableTracker.
 	// We don't need the explicitly set a channelInformer, as this will dynamically generate one for us.
 	// This code needs to be called before checking the existence of the `channel`, in order to make sure the
@@ -346,13 +346,14 @@ func (r *Reconciler) trackAndFetchChannel(ctx context.Context, sub *v1alpha1.Sub
 // and verifies it's a channelable (so that we can operate on it via patches).
 // If the Channel is a channels.messaging type (hence, it's only a factory for
 // underlying channels), fetch and validate the "backing" channel.
-func (r *Reconciler) getChannel(ctx context.Context, sub *v1alpha1.Subscription) (*eventingduckv1alpha1.ChannelableCombined, pkgreconciler.Event) {
+func (r *Reconciler) getChannel(ctx context.Context, sub *v1beta1.Subscription) (*eventingduckv1alpha1.ChannelableCombined, pkgreconciler.Event) {
 	logging.FromContext(ctx).Info("Getting channel", zap.Any("channel", sub.Spec.Channel))
 
 	// 1. Track the channel pointed by subscription.
 	//   a. If channel is a Channel.messaging.knative.dev
 	obj, err := r.trackAndFetchChannel(ctx, sub, sub.Spec.Channel)
 	if err != nil {
+		logging.FromContext(ctx).Warn("failed", zap.Any("channel", sub.Spec.Channel), zap.Error(err))
 		return nil, err
 	}
 
@@ -361,7 +362,8 @@ func (r *Reconciler) getChannel(ctx context.Context, sub *v1alpha1.Subscription)
 	// Test to see if the channel is Channel.messaging because it is going
 	// to have a "backing" channel that is what we need to actually operate on
 	// as well as keep track of.
-	if channelGVK.Group == gvk.Group && channelGVK.Kind == gvk.Kind {
+	if (v1beta1ChannelGVK.Group == gvk.Group && v1beta1ChannelGVK.Kind == gvk.Kind) ||
+		(v1ChannelGVK.Group == gvk.Group && v1ChannelGVK.Kind == gvk.Kind) {
 		// Track changes on Channel.
 		// Ref: https://github.com/knative/eventing/issues/2641
 		// NOTE: There is a race condition with using the channelableTracker
@@ -372,17 +374,17 @@ func (r *Reconciler) getChannel(ctx context.Context, sub *v1alpha1.Subscription)
 		// re-sync therefore we have to track Channels using a tracker linked
 		// to the cache we intend to use to pull the Channel from. This linkage
 		// is setup in NewController for r.tracker.
-		apiVersion, kind := gvk.ToAPIVersionAndKind()
 		if err := r.tracker.TrackReference(tracker.Reference{
-			APIVersion: apiVersion,
-			Kind:       kind,
+			APIVersion: "messaging.knative.dev/v1beta1",
+			Kind:       "Channel",
 			Namespace:  sub.Namespace,
 			Name:       sub.Spec.Channel.Name,
 		}, sub); err != nil {
+			logging.FromContext(ctx).Info("TrackReference for Channel failed", zap.Any("channel", sub.Spec.Channel), zap.Error(err))
 			return nil, err
 		}
 
-		logging.FromContext(ctx).Warn("fetching backing channel", zap.Any("channel", sub.Spec.Channel))
+		logging.FromContext(ctx).Debug("fetching backing channel", zap.Any("channel", sub.Spec.Channel))
 		// Because the above (trackAndFetchChannel) gives us back a Channelable
 		// the status of it will not have the extra bits we need (namely, pointer
 		// and status of the actual "backing" channel), we fetch it using typed
@@ -394,10 +396,11 @@ func (r *Reconciler) getChannel(ctx context.Context, sub *v1alpha1.Subscription)
 
 		if !channel.Status.IsReady() || channel.Status.Channel == nil {
 			logging.FromContext(ctx).Warn("backing channel not ready", zap.Any("channel", sub.Spec.Channel), zap.Any("backing channel", channel))
-			return nil, pkgreconciler.NewEvent(corev1.EventTypeWarning, "ChannelNotReady", "Backing channel is not ready")
+			return nil, fmt.Errorf("channel is not ready.")
 		}
 
-		obj, err = r.trackAndFetchChannel(ctx, sub, *channel.Status.Channel)
+		statCh := corev1.ObjectReference{Name: channel.Status.Channel.Name, Namespace: sub.Namespace, Kind: channel.Status.Channel.Kind, APIVersion: channel.Status.Channel.APIVersion}
+		obj, err = r.trackAndFetchChannel(ctx, sub, statCh)
 		if err != nil {
 			return nil, err
 		}
@@ -407,22 +410,10 @@ func (r *Reconciler) getChannel(ctx context.Context, sub *v1alpha1.Subscription)
 	ch, ok := obj.(*eventingduckv1alpha1.ChannelableCombined)
 	if !ok {
 		logging.FromContext(ctx).Error("Failed to convert to Channelable Object", zap.Any("channel", sub.Spec.Channel), zap.Error(err))
-		return nil, err
+		return nil, fmt.Errorf("Failed to convert to Channelable Object: %+v", obj)
 	}
 
-	retCh := ch.DeepCopy()
-	gvk = retCh.GetObjectKind().GroupVersionKind()
-	// IMC has been know to lie about the duck version it supports. We know that
-	// v1alpha1 supports v1alpha1 Subscribable duck so override it here.
-	// If there are other channels that have this lying behaviour, add them here...
-	if gvk.Kind == "InMemoryChannel" && gvk.Version == "v1alpha1" {
-		if retCh.Annotations == nil {
-			retCh.Annotations = make(map[string]string)
-		}
-		retCh.Annotations[messaging.SubscribableDuckVersionAnnotation] = "v1alpha1"
-	}
-
-	return retCh, nil
+	return ch.DeepCopy(), nil
 }
 
 func isNilOrEmptyDeliveryDeadLetterSink(delivery *eventingduckv1beta1.DeliverySpec) bool {
@@ -434,7 +425,7 @@ func isNilOrEmptyDestination(destination *duckv1.Destination) bool {
 	return destination == nil || equality.Semantic.DeepEqual(destination, &duckv1.Destination{})
 }
 
-func (r *Reconciler) syncPhysicalChannel(ctx context.Context, sub *v1alpha1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined, isDeleted bool) (bool, error) {
+func (r *Reconciler) syncPhysicalChannel(ctx context.Context, sub *v1beta1.Subscription, channel *eventingduckv1alpha1.ChannelableCombined, isDeleted bool) (bool, error) {
 	logging.FromContext(ctx).Debug("Reconciling physical from Channel", zap.Any("sub", sub))
 	if patched, patchErr := r.patchSubscription(ctx, sub.Namespace, channel, sub); patchErr != nil {
 		if isDeleted && apierrors.IsNotFound(patchErr) {
@@ -447,7 +438,7 @@ func (r *Reconciler) syncPhysicalChannel(ctx context.Context, sub *v1alpha1.Subs
 	}
 }
 
-func (r *Reconciler) patchSubscription(ctx context.Context, namespace string, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) (bool, error) {
+func (r *Reconciler) patchSubscription(ctx context.Context, namespace string, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) (bool, error) {
 	after := channel.DeepCopy()
 
 	if sub.DeletionTimestamp.IsZero() {
@@ -476,13 +467,14 @@ func (r *Reconciler) patchSubscription(ctx context.Context, namespace string, ch
 		logging.FromContext(ctx).Warn("Failed to patch the Channel", zap.Error(err), zap.Any("patch", patch))
 		return false, err
 	}
-	logging.FromContext(ctx).Debug("Patched resource", zap.Any("patched", patched))
+	logging.FromContext(ctx).Debug("Patched resource", zap.Any("patch", patch), zap.Any("patched", patched))
 	return true, nil
 }
 
-func (r *Reconciler) updateChannelRemoveSubscription(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) {
+func (r *Reconciler) updateChannelRemoveSubscription(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) {
 	if channel.Annotations != nil {
-		if channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1beta1" {
+		if channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1beta1" ||
+			channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1" {
 			r.updateChannelRemoveSubscriptionV1Beta1(ctx, channel, sub)
 			return
 		}
@@ -490,7 +482,7 @@ func (r *Reconciler) updateChannelRemoveSubscription(ctx context.Context, channe
 	r.updateChannelRemoveSubscriptionV1Alpha1(ctx, channel, sub)
 }
 
-func (r *Reconciler) updateChannelRemoveSubscriptionV1Beta1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) {
+func (r *Reconciler) updateChannelRemoveSubscriptionV1Beta1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) {
 	for i, v := range channel.Spec.Subscribers {
 		if v.UID == sub.UID {
 			channel.Spec.Subscribers = append(
@@ -501,7 +493,7 @@ func (r *Reconciler) updateChannelRemoveSubscriptionV1Beta1(ctx context.Context,
 	}
 }
 
-func (r *Reconciler) updateChannelRemoveSubscriptionV1Alpha1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) {
+func (r *Reconciler) updateChannelRemoveSubscriptionV1Alpha1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) {
 	if channel.Spec.Subscribable == nil {
 		return
 	}
@@ -516,9 +508,10 @@ func (r *Reconciler) updateChannelRemoveSubscriptionV1Alpha1(ctx context.Context
 	}
 }
 
-func (r *Reconciler) updateChannelAddSubscription(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) {
+func (r *Reconciler) updateChannelAddSubscription(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) {
 	if channel.Annotations != nil {
-		if channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1beta1" {
+		if channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1beta1" ||
+			channel.Annotations[messaging.SubscribableDuckVersionAnnotation] == "v1" {
 			r.updateChannelAddSubscriptionV1Beta1(ctx, channel, sub)
 			return
 		}
@@ -526,7 +519,7 @@ func (r *Reconciler) updateChannelAddSubscription(ctx context.Context, channel *
 	r.updateChannelAddSubscriptionV1Alpha1(ctx, channel, sub)
 }
 
-func (r *Reconciler) updateChannelAddSubscriptionV1Alpha1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) {
+func (r *Reconciler) updateChannelAddSubscriptionV1Alpha1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) {
 	if channel.Spec.Subscribable == nil {
 		channel.Spec.Subscribable = &eventingduckv1alpha1.Subscribable{
 			Subscribers: []eventingduckv1alpha1.SubscriberSpec{{
@@ -535,7 +528,6 @@ func (r *Reconciler) updateChannelAddSubscriptionV1Alpha1(ctx context.Context, c
 				SubscriberURI:     sub.Status.PhysicalSubscription.SubscriberURI,
 				ReplyURI:          sub.Status.PhysicalSubscription.ReplyURI,
 				DeadLetterSinkURI: sub.Status.PhysicalSubscription.DeadLetterSinkURI,
-				// TODO: delivery?
 			}},
 		}
 		return
@@ -548,7 +540,6 @@ func (r *Reconciler) updateChannelAddSubscriptionV1Alpha1(ctx context.Context, c
 			channel.Spec.Subscribable.Subscribers[i].SubscriberURI = sub.Status.PhysicalSubscription.SubscriberURI
 			channel.Spec.Subscribable.Subscribers[i].ReplyURI = sub.Status.PhysicalSubscription.ReplyURI
 			channel.Spec.Subscribable.Subscribers[i].DeadLetterSinkURI = sub.Status.PhysicalSubscription.DeadLetterSinkURI
-			// TODO: delivery?
 			return
 		}
 	}
@@ -561,39 +552,45 @@ func (r *Reconciler) updateChannelAddSubscriptionV1Alpha1(ctx context.Context, c
 			SubscriberURI:     sub.Status.PhysicalSubscription.SubscriberURI,
 			ReplyURI:          sub.Status.PhysicalSubscription.ReplyURI,
 			DeadLetterSinkURI: sub.Status.PhysicalSubscription.DeadLetterSinkURI,
-			// TODO: delivery?
 		})
 }
 
-func (r *Reconciler) updateChannelAddSubscriptionV1Beta1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1alpha1.Subscription) {
+func (r *Reconciler) updateChannelAddSubscriptionV1Beta1(ctx context.Context, channel *eventingduckv1alpha1.ChannelableCombined, sub *v1beta1.Subscription) {
 	// Look to update subscriber.
 	for i, v := range channel.Spec.Subscribers {
 		if v.UID == sub.UID {
 			channel.Spec.Subscribers[i].Generation = sub.Generation
 			channel.Spec.Subscribers[i].SubscriberURI = sub.Status.PhysicalSubscription.SubscriberURI
 			channel.Spec.Subscribers[i].ReplyURI = sub.Status.PhysicalSubscription.ReplyURI
-			channel.Spec.Subscribers[i].Delivery = &eventingduckv1beta1.DeliverySpec{
-				DeadLetterSink: &duckv1.Destination{
-					URI: sub.Status.PhysicalSubscription.DeadLetterSinkURI,
-				},
+			// Only set the deadletter sink if it's not nil. Otherwise we'll just end up patching
+			// empty delivery in there.
+			if sub.Status.PhysicalSubscription.DeadLetterSinkURI != nil {
+				channel.Spec.Subscribers[i].Delivery = &eventingduckv1beta1.DeliverySpec{
+					DeadLetterSink: &duckv1.Destination{
+						URI: sub.Status.PhysicalSubscription.DeadLetterSinkURI,
+					},
+				}
 			}
-			// TODO: delivery?
 			return
 		}
 	}
 
-	// Must not have been found. Add it.
-	channel.Spec.Subscribers = append(channel.Spec.Subscribers,
-		eventingduckv1beta1.SubscriberSpec{
-			UID:           sub.UID,
-			Generation:    sub.Generation,
-			SubscriberURI: sub.Status.PhysicalSubscription.SubscriberURI,
-			ReplyURI:      sub.Status.PhysicalSubscription.ReplyURI,
-			Delivery: &eventingduckv1beta1.DeliverySpec{
-				DeadLetterSink: &duckv1.Destination{
-					URI: sub.Status.PhysicalSubscription.DeadLetterSinkURI,
-				},
+	toAdd := eventingduckv1beta1.SubscriberSpec{
+		UID:           sub.UID,
+		Generation:    sub.Generation,
+		SubscriberURI: sub.Status.PhysicalSubscription.SubscriberURI,
+		ReplyURI:      sub.Status.PhysicalSubscription.ReplyURI,
+	}
+	// Only set the deadletter sink if it's not nil. Otherwise we'll just end up patching
+	// empty delivery in there.
+	if sub.Status.PhysicalSubscription.DeadLetterSinkURI != nil {
+		toAdd.Delivery = &eventingduckv1beta1.DeliverySpec{
+			DeadLetterSink: &duckv1.Destination{
+				URI: sub.Status.PhysicalSubscription.DeadLetterSinkURI,
 			},
-			// TODO: delivery?
-		})
+		}
+	}
+
+	// Must not have been found. Add it.
+	channel.Spec.Subscribers = append(channel.Spec.Subscribers, toAdd)
 }
